//
// Author: E.Gorini
// last modification 31/10/2014
//
//   to execute on a data file with x,y,dy columns:
//   [0] .x LinFit.C("nomefile.dat");
//
#include <TH1>
#include <TF1>
#include <TLegend>
#include <TCanvas>
#include <Riostream>
#include <vector>

void LinFit(string file){

// ROOT initialization
  gROOT->Reset();
  gStyle->SetOptStat(1);
  gStyle->SetOptFit(111111);


// here opens the data file
  ifstream in;
  in.open(file.c_str());
// get out if file do not exists..
  if(in.is_open()==0){
     in.close();
     gROOT->Reset();
    cout << "File " << file <<" do not exist, exit\n";
    break;
  }
//
// File exists, start reading the info part
//
// reads plot title and X and Y labels
  string Title,Xtitle,Ytitle;
  getline(in,Title); Title.substr(2,Title.length());
  getline(in,Xtitle); Xtitle.substr(2,Xtitle.length());
  getline(in,Ytitle); Ytitle.substr(2,Ytitle.length());
  cout << "\n\tPlot Title is: \""<< Title <<"\", \n";
  cout << "\tx axis title is: \""<< Xtitle <<"\", \n";
  cout << "\ty axis title is: \""<< Ytitle <<"\".\n";
// reads parameter names
  vector<string> parnames;
  string sbuf;
  in >> sbuf;
  for (int np=0;np<2;np++){
    in >> sbuf;
    parnames.push_back(sbuf);
  }
// reads type of data
  TString flag;
  in >> sbuf; in >> flag;
// close file
  in.close();
// reopen for data reading
  in.open(file.c_str());
  TString FileName=file;
  cout << "\nReading Data... \n-------------\n";

// create and set canvas style
  TCanvas *c1 = new TCanvas("c1",Title.c_str(),10,10,900,700);
  c1->SetFillColor(19);
  c1->SetGrid(1,1);

// plot the measurements reading from file
  if    (flag=="noerr")gr = new TGraph(FileName);
  elseif(flag=="yerr" )gr = new TGraphErrors(FileName,"\%lg \%lg \%lg");
  elseif(flag=="xyerr")gr = new TGraphErrors(FileName);
  else{
    cout << " wrong flag, exit \n" ;
    return;
  }

// set plot attributes
  gr->SetTitle(Title.c_str());
  gr->SetMarkerColor(kBlue);
  gr->SetMarkerStyle(21);
  gr->GetXaxis()->SetTitle(Xtitle.c_str());
  gr->GetYaxis()->SetTitle(Ytitle.c_str());

// draw plot
  gr->Draw("AP");
  c1->Update();

// create the linear function to be fitted
  TF1 *fff = new TF1("fff","[0]+[1]*x");
// set parameter names
  for (int np=0;np<2;np++)fff->SetParName(np,parnames[np].c_str());
// fit and plot
  gr->Fit("fff","Q");
  c1->Update();

// get fit results and print with better precision
  TF1 *fr=gr->GetFunction("fff");
  cout << "\nFit Results\n" << "-----------\n";
  cout << parnames[0] << " = " << fr->GetParameter(0) << " +/- " << fr->GetParError(0) <<"\n";
  cout << parnames[1] << " = " << fr->GetParameter(1) << " +/- " << fr->GetParError(1) <<"\n\n";

// save the plots in pdf with the same name of the original Input File
  FileName.Remove(FileName.Length()-4,FileName.Length());
  c1->Print(FileName+".pdf");
}

